@@ -20,7 +20,8 @@ This module provides the following:
 References
 ----------
 
-HepPDT and HepPID versions 3.04.01: http://lcgapp.cern.ch/project/simu/HepPDT/
+- PDG document "Monte Carlo Particle Numbering Scheme".
+- HepPDT and HepPID versions 3.04.01.
 """
 
 from __future__ import print_function, division, absolute_import
@@ -51,12 +52,14 @@ class Location(IntEnum):
 def is_valid(pdgid):
     # type: (PDGID_TYPE) -> bool
     """Is it a valid PDG ID?"""
-    if _fundamental_id(pdgid) > 0:
+    if _fundamental_id(pdgid) != 0:  # function always returns a number >= 0
         return True
     if is_meson(pdgid):
         return True
     if is_baryon(pdgid):
         return True
+    if is_gauge_boson_or_higgs(pdgid):
+        return True
     if is_pentaquark(pdgid):
         return True
     if is_SUSY(pdgid):
@@ -69,12 +72,14 @@ def is_valid(pdgid):
         return True
     if is_pentaquark(pdgid):
         return True
+    if is_generator_specific(pdgid):
+        return True
+    if is_technicolor(pdgid):
+        return True
+    if is_composite_quark_or_lepton(pdgid):
+        return True
     if _extra_bits(pdgid) > 0:
-        if is_nucleus(pdgid):
-            return True
-        if is_Qball(pdgid):
-            return True
-        return False
+        return is_Qball(pdgid) or is_nucleus(pdgid)
     return False
 
 
@@ -84,12 +89,22 @@ def abspid(pdgid):
     return abs(int(pdgid))
 
 
+def is_quark(pdgid):
+    # type: (PDGID_TYPE) -> bool
+    """
+    Does this PDG ID correspond to a quark?
+
+    Fourth-generation quarks are included, but not excited (composite) quarks.
+    """
+    return 1 <= abspid(pdgid) <= 8
+
+
 def is_lepton(pdgid):
     # type: (PDGID_TYPE) -> bool
     """Does this PDG ID correspond to a lepton?"""
     if _extra_bits(pdgid) > 0:
         return False
-    if _fundamental_id(pdgid) >= 11 and _fundamental_id(pdgid) <= 18:
+    if 11 <= int(_fundamental_id(pdgid)) <= 18:
         return True
     return False
 
@@ -99,7 +114,7 @@ def is_hadron(pdgid):
     """Does this PDG ID correspond to a hadron?"""
     # Special case of proton and neutron:
     # needs to be checked first since _extra_bits(pdgid) > 0 for nuclei
-    if abs(int(pdgid)) in (1000000010, 1000010010):
+    if abs(int(pdgid)) in {1000000010, 1000010010}:
         return True
     if _extra_bits(pdgid) > 0:
         return False
@@ -109,6 +124,8 @@ def is_hadron(pdgid):
         return True
     if is_pentaquark(pdgid):
         return True
+    if is_Rhadron(pdgid):
+        return True
     return False
 
 
@@ -119,13 +136,16 @@ def is_meson(pdgid):
         return False
     if abspid(pdgid) <= 100:
         return False
-    if _fundamental_id(pdgid) <= 100 and _fundamental_id(pdgid) > 0:
+    if 0 < int(_fundamental_id(pdgid)) <= 100:
         return False
-    if abspid(pdgid) in (130, 210, 310):
+    # Special IDs - K(L)0, ???, K(S)0
+    if abspid(pdgid) in {130, 210, 310}:
         return True
-    if abspid(pdgid) in (150, 350, 510, 530):
+    # Special IDs - B(L)0, B(sL)0, B(H)0, B(sH)0
+    if abspid(pdgid) in {150, 350, 510, 530}:
         return True
-    if int(pdgid) in (110, 990, 9990):
+    # Special particles - reggeon, pomeron, odderon
+    if int(pdgid) in {110, 990, 9990}:
         return True
     if (
         _digit(pdgid, Location.Nj) > 0
@@ -151,14 +171,19 @@ def is_baryon(pdgid):
         return False
     # Special case of proton and neutron:
     # needs to be checked first since _extra_bits(pdgid) > 0 for nuclei
-    if abs(int(pdgid)) in (1000000010, 1000010010):
+    if abs(int(pdgid)) in {1000000010, 1000010010}:
         return True
+
     if _extra_bits(pdgid) > 0:
         return False
-    if _fundamental_id(pdgid) > 0 and _fundamental_id(pdgid) <= 100:
+
+    if 0 < _fundamental_id(pdgid) <= 100:
         return False
-    if abspid(pdgid) == 2110 or abspid(pdgid) == 2210:
+
+    # Old codes for diffractive p and n (MC usage)
+    if abspid(pdgid) in {2110, 2210}:
         return True
+
     if (
         _digit(pdgid, Location.Nj) > 0
         and _digit(pdgid, Location.Nq3) > 0
@@ -166,8 +191,10 @@ def is_baryon(pdgid):
         and _digit(pdgid, Location.Nq1) > 0
     ):
         return True
+
     if is_Rhadron(pdgid) or is_pentaquark(pdgid):
         return False
+
     return False
 
 
@@ -178,7 +205,7 @@ def is_diquark(pdgid):
         return False
     if abspid(pdgid) <= 100:
         return False
-    if _fundamental_id(pdgid) <= 100 and _fundamental_id(pdgid) > 0:
+    if 0 < int(_fundamental_id(pdgid)) <= 100:
         return False
     if (
         _digit(pdgid, Location.Nj) > 0
@@ -204,7 +231,7 @@ def is_nucleus(pdgid):
     # A proton can be a Hydrogen nucleus
     # A neutron can be considered as a nucleus when given the PDG ID 1000000010,
     # hence consistency demands that is_nucleus(neutron) is True
-    if abspid(pdgid) in (2112, 2212):
+    if abspid(pdgid) in {2112, 2212}:
         return True
     if _digit(pdgid, Location.N10) == 1 and _digit(pdgid, Location.N9) == 0:
         # Charge should always be less than or equal to the baryon number
@@ -252,9 +279,80 @@ def is_pentaquark(pdgid):
     return True
 
 
+def is_gauge_boson_or_higgs(pdgid):
+    # type: (PDGID_TYPE) -> bool
+    """
+    Does this PDG ID correspond to a gauge boson or a Higgs?
+
+    Codes 21-30 are reserved for the Standard Model gauge bosons and the Higgs.
+    The graviton and the boson content of a two-Higgs-doublet scenario
+    and of additional SU(2)xU(1) groups are found in the range 31-40.
+    """
+    return True if 21 <= abspid(pdgid) <= 40 else False
+
+
+def is_sm_gauge_boson_or_higgs(pdgid):
+    # type: (PDGID_TYPE) -> bool
+    """
+    Does this PDG ID correspond to a Standard Model gauge boson or Higgs?
+
+    Codes 21-30 are reserved for the Standard Model gauge bosons and the Higgs,
+    but only the codes 21-25 actually correspond to SM particles.
+    """
+    if abspid(pdgid) == 24:  # W is the only SM gauge boson not its antiparticle
+        return True
+
+    return True if 21 <= int(pdgid) <= 25 else False
+
+
+def is_generator_specific(pdgid):
+    # type: (PDGID_TYPE) -> bool
+    """
+    Does this PDG ID correspond to generator-specific pseudoparticles or concepts?
+
+    Codes 81-100 are reserved for generator-specific pseudoparticles and concepts.
+    Codes 901-930, 1901-1930, 2901-2930, and 3901-3930 are for
+    additional components of Standard Model parton distribution functions,
+    where the latter three ranges are intended to distinguish
+    left/right/longitudinal components.
+    Codes 998 and 999 are reserved for GEANT tracking purposes.
+    """
+    aid = abspid(pdgid)
+    if 81 <= aid <= 100:
+        return True
+    if 901 <= aid <= 930:
+        return True
+    if 1901 <= aid <= 1930:
+        return True
+    if 2901 <= aid <= 2930:
+        return True
+    if 3901 <= aid <= 3930:
+        return True
+    if aid in {998, 999}:
+        return True
+    if aid in {20022, 480000000}:  # Special cases of opticalphoton and geantino
+        return True
+    return False
+
+
+def is_special_particle(pdgid):
+    # type: (PDGID_TYPE) -> bool
+    """
+    Does this PDG ID correspond to a special particle?
+
+    Special particle in the sense of the classification in the PDG MC particle numbering scheme document,
+    hence the graviton, the DM (S = 0, 1/2, 1) particles, the reggeons (reggeon, pomeron and odderon),
+    and all generator-specific pseudo-particles and concepts, see `is_generator_specific`.
+    """
+    return pdgid in {39, 41, 42, 51, 52, 53, 110, 990, 9990} or is_generator_specific(
+        pdgid
+    )
+
+
 def is_Rhadron(pdgid):
     # type: (PDGID_TYPE) -> bool
-    """Does this PDG ID correspond to an R-hadron?
+    """
+    Does this PDG ID correspond to an R-hadron?
 
     An R-hadron is of the form 10abcdj, 100abcj, or 1000abj,
     where j = 2J + 1 gives the spin; b, c, and d are quarks or gluons;
@@ -317,7 +415,7 @@ def is_dyon(pdgid):
         return False
     if _digit(pdgid, Location.Nr) != 1:
         return False
-    if _digit(pdgid, Location.Nl) != 1 and _digit(pdgid, Location.Nl) != 2:
+    if _digit(pdgid, Location.Nl) not in {1, 2}:
         return False
     if _digit(pdgid, Location.Nq3) == 0:
         return False
@@ -344,6 +442,34 @@ def is_SUSY(pdgid):
     return True
 
 
+def is_technicolor(pdgid):
+    # type: (PDGID_TYPE) -> bool
+    """
+    Does this PDG ID correspond to a Technicolor state?
+
+    Technicolor states have N = 3.
+    """
+    if _extra_bits(pdgid) > 0:
+        return False
+    return True if _digit(pdgid, Location.N) == 3 else False
+
+
+def is_composite_quark_or_lepton(pdgid):
+    # type: (PDGID_TYPE) -> bool
+    """
+    Does this PDG ID correspond to an excited (composite) quark or lepton?
+
+    Excited (composite) quarks and leptons have N = 4 and Nr = 0.
+    """
+    if _extra_bits(pdgid) > 0:
+        return False
+    if _fundamental_id(pdgid) == 0:
+        return False
+    if not (_digit(pdgid, Location.N) == 4 and _digit(pdgid, Location.Nr) == 0):
+        return False
+    return True
+
+
 def has_down(pdgid):
     # type: (PDGID_TYPE) -> bool
     """Does this particle contain a down quark?"""
@@ -382,15 +508,28 @@ def has_top(pdgid):
 
 def has_fundamental_anti(pdgid):
     # type: (PDGID_TYPE) -> bool
-    """If this is a fundamental particle, does it have a valid antiparticle?"""
-    # These are defined by the generator and therefore are always valid
-    fid = _fundamental_id(pdgid)
-    if fid in range(80, 101):
-        return True
+    """
+    If this is a fundamental particle, does it have a valid antiparticle?
+
+    Notes
+    -----
+    Based on the current list of defined particles/concepts
+    in the PDG Monte Carlo Particle Numbering Scheme document.
+    """
+    fid = _fundamental_id(pdgid)  # always a positive integer
+
+    # Check generator-specific PDGIDs
+    if 81 <= fid <= 100:
+        return True if fid in {82, 84, 85, 86, 87} else False
+
     # Check PDGIDs from 1 to 79
-    _cp_conjugates = (21, 22, 23, 25, 32, 33, 35, 36, 39, 41)
-    if fid in range(1, 80) and fid not in _cp_conjugates and is_valid(abs(int(pdgid))):
-        return True
+    _cp_conjugates = {21, 22, 23, 25, 32, 33, 35, 36, 39, 40, 43}
+    _unassigned = (
+        [9, 10, 19, 20, 26] + list(range(26, 32)) + list(range(45, 80))
+    )  # not in conversion.csv
+    if (1 <= fid <= 79) and fid not in _cp_conjugates:
+        return False if fid in _unassigned else True
+
     return False
 
 
@@ -543,9 +682,9 @@ def three_charge(pdgid):
         # the charge sign will be changed below if pid < 0
         if _digit(pdgid, Location.Nl) == 2:
             charge = -charge
-    elif sid > 0 and sid <= 100:  # use table
+    elif 0 < sid <= 100:  # use table
         charge = ch100[sid - 1]
-        if aid in (1000017, 1000018, 1000034, 1000052, 1000053, 1000054):
+        if aid in {1000017, 1000018, 1000034, 1000052, 1000053, 1000054}:
             charge = 0
         if aid == 5100061 or aid == 5100062:
             charge = 6
@@ -575,20 +714,20 @@ def j_spin(pdgid):
         return None
     if _fundamental_id(pdgid) > 0:
         fund = _fundamental_id(pdgid)
-        if fund > 0 and fund < 7:
+        if 0 < fund < 7:
             return 2  # 4th generation quarks not dealt with !
         if fund == 9:
             return 3
-        if fund > 10 and fund < 17:
+        if 10 < fund < 17:
             return 2  # 4th generation leptons not dealt with !
-        if fund > 20 and fund < 25:
+        if 20 < fund < 25:
             return 3
         return None
-    elif abs(int(pdgid)) in (1000000010, 1000010010):  # neutron, proton
+    elif abs(int(pdgid)) in {1000000010, 1000010010}:  # neutron, proton
         return 2
     elif _extra_bits(pdgid) > 0:
         return None
-    if pdgid in (130, 310):
+    if pdgid in {130, 310}:
         return 1  # Special cases of the KS and KL !
     return abspid(pdgid) % 10
 
@@ -615,26 +754,27 @@ def S(pdgid):
     """
     if not is_meson(pdgid):
         return None
+
     if not is_valid(pdgid):
         return None
 
-    nl = (abspid(pdgid) // 10000) % 10
-    js = abspid(pdgid) % 10
     if (abspid(pdgid) // 1000000) % 10 == 9:
         return None  # no knowledge so far
-    if nl == 0 and js >= 3:
-        return 1
-    elif nl == 0 and js == 1:
+
+    nl = (abspid(pdgid) // 10000) % 10
+    js = abspid(pdgid) % 10
+
+    if not (js == 1 or js >= 3):
         return 0
-    elif nl == 1 and js >= 3:
+
+    if nl == 0:
+        return 0 if js == 1 else 1
+    elif nl == 1:
+        return 1 if js == 1 else 0
+    elif nl in {2, 3}:
+        return 1 if js >= 3 else 0
+    else:
         return 0
-    elif nl == 2 and js >= 3:
-        return 1
-    elif nl == 1 and js == 1:
-        return 1
-    elif nl == 3 and js >= 3:
-        return 1
-    return 0
 
 
 def s_spin(pdgid):
@@ -665,50 +805,57 @@ def L(pdgid):
     """
     if not is_meson(pdgid):
         return None
+
     if not is_valid(pdgid):
         return None
 
-    nl = (abspid(pdgid) // 10000) % 10
-    js = abspid(pdgid) % 10
     if (abspid(pdgid) // 1000000) % 10 == 9:
         return None  # no knowledge so far
 
-    if nl == 0 and js == 3:
-        return 0
-    elif nl == 0 and js == 5:
-        return 1
-    elif nl == 0 and js == 7:
-        return 2
-    elif nl == 0 and js == 9:
-        return 3
-    elif nl == 0 and js == 1:
-        return 0
-    elif nl == 1 and js == 3:
-        return 1
-    elif nl == 1 and js == 5:
-        return 2
-    elif nl == 1 and js == 7:
-        return 3
-    elif nl == 1 and js == 9:
-        return 4
-    elif nl == 2 and js == 3:
-        return 1
-    elif nl == 2 and js == 5:
-        return 2
-    elif nl == 2 and js == 7:
-        return 3
-    elif nl == 2 and js == 9:
-        return 4
-    elif nl == 1 and js == 1:
-        return 1
-    elif nl == 3 and js == 3:
-        return 2
-    elif nl == 3 and js == 5:
-        return 3
-    elif nl == 3 and js == 7:
-        return 4
-    elif nl == 3 and js == 9:
-        return 5
+    nl = (abspid(pdgid) // 10000) % 10
+    js = abspid(pdgid) % 10
+
+    if nl == 0:
+        if js == 1:
+            return 0
+        if js == 3:
+            return 0
+        if js == 5:
+            return 1
+        if js == 7:
+            return 2
+        if js == 9:
+            return 3
+    elif nl == 1:
+        if js == 1:
+            return 1
+        if js == 3:
+            return 1
+        if js == 5:
+            return 2
+        if js == 7:
+            return 3
+        if js == 9:
+            return 4
+    elif nl == 2:
+        if js == 3:
+            return 1
+        if js == 5:
+            return 2
+        if js == 7:
+            return 3
+        if js == 9:
+            return 4
+    elif nl == 3:
+        if js == 3:
+            return 2
+        if js == 5:
+            return 3
+        if js == 7:
+            return 4
+        if js == 9:
+            return 5
+
     return 0
 
 
@@ -733,7 +880,7 @@ def A(pdgid):
     # A proton can be a Hydrogen nucleus
     # A neutron can be considered as a nucleus when given the PDG ID 1000000010,
     # hence consistency demands that A(neutron) = 1
-    if abspid(pdgid) in (2112, 2212):
+    if abspid(pdgid) in {2112, 2212}:
         return 1
     if _digit(pdgid, Location.N10) != 1 or _digit(pdgid, Location.N9) != 0:
         return None
@@ -781,6 +928,10 @@ def _fundamental_id(pdgid):
     Returns 0 if the particle is not fundamental or not standard (PDG ID with more than 7 digits).
 
     PDGID=100 is a special case (internal generator ID's are 81-100).
+
+    Notes
+    -----
+    Function always returns a number >= 0.
     """
     if _extra_bits(pdgid) > 0:
         return 0
@@ -797,25 +948,29 @@ def _has_quark_q(pdgid, q):
     """
     Helper function - does this particle contain a quark q?
 
-    Note that q is always positive, so [1, 6] for Standard Model quarks.
+    Note that q is always positive, so [1, 6] for Standard Model quarks
+    and [7, 8] for fourth-generation quarks.
     """
     # Nuclei can also contain strange quarks,
     # cf. the definition of a nucleus PDG ID in is_nucleus.
     # This check needs to be done first since _extra_bits(pdgid) > 0 for nuclei
     if is_nucleus(pdgid):
-        if q in (1, 2):
+        if q in {1, 2}:
             return True  # Nuclei by construction contain up and down quarks
-        elif q == 3 and pdgid not in (2112, 2212):
+        elif q == 3 and pdgid not in {2112, 2212}:
             if _digit(pdgid, Location.N8) > 0:
                 return True
             else:
                 return False
+
     if _extra_bits(pdgid) > 0:
         return False
     if _fundamental_id(pdgid) > 0:
         return False
+
     if is_dyon(pdgid):
         return False
+
     if is_Rhadron(pdgid):
         iz = 7
         for loc in range(6, 1, -1):
@@ -828,13 +983,16 @@ def _has_quark_q(pdgid, q):
                 if _digit(pdgid, loc) == q:
                     return True
         return False
+
     if (
         _digit(pdgid, Location.Nq3) == q
         or _digit(pdgid, Location.Nq2) == q
         or _digit(pdgid, Location.Nq1) == q
     ):
         return True
+
     if is_pentaquark(pdgid):
         if _digit(pdgid, Location.Nl) == q or _digit(pdgid, Location.Nr) == q:
             return True
+
     return False
